@@ -1,4 +1,25 @@
 # -*- coding: utf-8 -*-
+#
+#  reportapi/filters.py
+#  
+#  Copyright 2014 Grigoriy Kramarenko <root@rosix.ru>
+#  
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#  
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#  
+#  You should have received a copy of the GNU General Public License
+#  along with this program; if not, write to the Free Software
+#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+#  MA 02110-1301, USA.
+#  
+#  
 from __future__ import unicode_literals
 from django.utils.encoding import smart_text, python_2_unicode_compatible
 from django.utils import six
@@ -47,7 +68,7 @@ class BaseFilter(object):
     verbose_name = None
 
     def __str__(self):
-        return '<%s:%s>' % (self.__class__.__name__, self.name)
+        return '%s:%s' % (self.__class__.__name__, self.name)
 
     def __init__(self, name, required=False, conditions=None):
         if self.required is None:
@@ -60,10 +81,16 @@ class BaseFilter(object):
     def get_value(self, condition, value):
         return value
 
+    def get_value_range_label(self, condition, value):
+        if condition == 'range':
+            value = list(self.get_value_label(condition, value))
+            return value[0], value[-1] 
+        return None
+
     def get_value_label(self, condition, value):
         return self.get_value(condition, value)
 
-    def data(self, condition=None, value=None, inverse=False, **options):
+    def data(self, condition='truth', value=None, inverse=False, **options):
         """
         Метод получения информации об установленном фильтре.
         """
@@ -75,6 +102,8 @@ class BaseFilter(object):
         options['condition_label'] = _(condition)
         options['value'] = self.get_value(condition, value)
         options['value_label'] = self.get_value_label(condition, value)
+        if condition == 'range':
+            options['value_range_label'] = self.get_value_range_label(condition, value)
 
         return options
 
@@ -245,6 +274,8 @@ class FilterObject(BaseFilter):
             return value
         elif condition == 'exact':
             return self.objects.get(pk=value)
+        elif condition == 'range':
+            return self.objects.filter(pk__gte=min(value), pk__lte=max(value)).order_by('pk')
         else:
             return self.objects.filter(pk__in=list(value)).order_by('pk')
 
@@ -272,6 +303,8 @@ class FilterDateTime(BaseFilter):
                 value = None
         if value is None:
             raise ValueError('One or more values not valid in `%s` filter.' % smart_text(self))
+        if condition == 'range':
+            value = [min(value), max(value)]
         return value
 
 class FilterDate(BaseFilter):
@@ -279,7 +312,7 @@ class FilterDate(BaseFilter):
 
     def get_value(self, condition, value):
         if isinstance(value, six.string_types):
-            value = parse_datetime(value)
+            value = parse_date(value)
         elif isinstance(value, (list, tuple)):
             _value = list(set([ parse_date(v) for v in value ]))
             if len(_value) == len(value):
@@ -288,6 +321,8 @@ class FilterDate(BaseFilter):
                 value = None
         if value is None:
             raise ValueError('One or more values not valid in `%s` filter.' % smart_text(self))
+        if condition == 'range':
+            value = [min(value), max(value)]
         return value
 
 class FilterTime(BaseFilter):
@@ -296,7 +331,7 @@ class FilterTime(BaseFilter):
 
     def get_value(self, condition, value):
         if isinstance(value, six.string_types):
-            value = parse_datetime(value)
+            value = parse_time(value)
         elif isinstance(value, (list, tuple)):
             _value = list(set([ parse_time(v) for v in value ]))
             if len(_value) == len(value):
@@ -305,6 +340,8 @@ class FilterTime(BaseFilter):
                 value = None
         if value is None:
             raise ValueError('One or more values not valid in `%s` filter.' % smart_text(self))
+        if condition == 'range':
+            value = [min(value), max(value)]
         return value
 
 class FilterBoolean(BaseFilter):
@@ -326,8 +363,8 @@ class FilterChoice(BaseFilter):
     def __init__(self, name, options=None, **kwargs):
         if options:
             self._options = dict(options)
-        else:
-            self._options = self._options or {}
+        elif not hasattr(self, '_options'):
+            self._options = {}
         super(FilterChoice, self).__init__(name, **kwargs)
 
     @property
@@ -344,7 +381,10 @@ class FilterChoice(BaseFilter):
     def get_value(self, condition, value):
         dic = self._options
         if condition in ('range', 'in'):
-            return [ self.keytype(x) for x in set(list(value)) if self.keytype(x) in dic ]
+            value = [ self.keytype(x) for x in set(list(value)) if self.keytype(x) in dic ]
+            if condition == 'range':
+                value = [min(value), max(value)]
+            return value
         else:
             return self.keytype(value)
 
@@ -361,6 +401,46 @@ class FilterMonth(FilterChoice):
     placeholder = _('Select month')
     _options = MONTHS
 
+PERIODS = {
+        'today': _('Today'),
+        'tomorrow': _('Tomorrow'),
+        'tomorrow2': _('Day after tomorrow'),
+        'yesterday': _('Yesterday'),
+        'yesterday2': _('Two days ago'),
+
+        'next2days': _('Next two days'),
+        'next3days': _('Next three days'),
+        'last2days': _('Last two days'),
+        'last3days': _('Last three days'),
+
+        'week': _('Current week'),
+        'next_week': _('Next week'),
+        'previous_week': _('Previous week'),
+
+        'month': _('Current month'),
+        'next_month': _('Next month'),
+        'previous_month': _('Previous month'),
+
+        'quarter1': _('First quarter'),
+        'quarter2': _('Second quarter'),
+        'quarter3': _('Third quarter'),
+        'quarter4': _('Fourth quarter'),
+
+        'halfyear1': _('First half year'),
+        'halfyear2': _('Second half year'),
+
+        'year': _('Current year'),
+        'next_year': _('Next year'),
+        'previous_year': _('Previous year'),
+    }
+
+class FilterPeriod(FilterChoice):
+    _type = 'period'
+    placeholder = _('Select period')
+    conditions = ('exact',)
+    keytype = str
+    _options = PERIODS
+
 
 def _search_in_fields(queryset, fields, query):
     """ Фильтрация """
