@@ -1,19 +1,64 @@
 # -*- coding: utf-8 -*-
+#
+#  reportapi/models.py
+#  
+#  Copyright 2014 Grigoriy Kramarenko <root@rosix.ru>
+#  
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#  
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#  
+#  You should have received a copy of the GNU General Public License
+#  along with this program; if not, write to the Free Software
+#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+#  MA 02110-1301, USA.
+#  
+#  
+
 from __future__ import unicode_literals
-from django.utils.encoding import smart_text, python_2_unicode_compatible
+import os, re, hashlib, subprocess
+
+from django.utils.encoding import smart_text, force_text, python_2_unicode_compatible
+from django.utils.crypto import get_random_string
 from django.utils import six
 from django.db import models
 from django.db.models import Q, get_model
-from django.utils.translation import ugettext_noop, ugettext_lazy as _
+from django.utils.translation import ugettext_noop, ugettext, ugettext_lazy as _
 from django.utils import timezone
 from django.template import RequestContext, loader
 from django.template.defaultfilters import slugify
 from django.core.files.base import ContentFile
 
-from reportapi.conf import (settings, REPORTAPI_CODE_HASHLIB,
-    REPORTAPI_UPLOAD_HASHLIB, REPORTAPI_FILES_UNIDECODE,
-    AUTH_USER_MODEL, REPORTAPI_CONVERTOR_BACKEND,
-    REPORTAPI_PDFCONVERT_ARGS1, REPORTAPI_PDFCONVERT_ARGS2)
+from reportapi.conf import (
+    settings,
+    REPORTAPI_CODE_HASHLIB,
+    REPORTAPI_UPLOAD_HASHLIB,
+    REPORTAPI_FILES_UNIDECODE,
+    AUTH_USER_MODEL,
+    REPORTAPI_UNOCONV_TO_ODF,
+    REPORTAPI_UNOCONV_TO_PDF,
+    REPORTAPI_BRAND_TEXT,
+    Header, Footer, Page
+)
+
+UNOCONV_EXE = "unoconv"
+
+if REPORTAPI_UNOCONV_TO_ODF or REPORTAPI_UNOCONV_TO_PDF:
+    import subprocess
+
+    try:
+        UNOCONV_EXE = subprocess.check_output(["which", "unoconv"]).replace('\n', '')
+    except:
+        REPORTAPI_UNOCONV_TO_ODF = False
+        REPORTAPI_UNOCONV_TO_PDF = False
+
+from reportapi.fields import JSONField
 
 User = get_model(*AUTH_USER_MODEL.split('.'))
 Group = User.groups.field.rel.to
@@ -24,36 +69,24 @@ if REPORTAPI_FILES_UNIDECODE:
 else:
     prep_filename = lambda x: x
 
-import os, re, hashlib, subprocess
-
 REPORTAPI_CODE_LENGTH = len(hashlib.new(REPORTAPI_CODE_HASHLIB).hexdigest())
 
-if REPORTAPI_CONVERTOR_BACKEND:
-    from django.utils.importlib import import_module
-    backend = REPORTAPI_CONVERTOR_BACKEND.split('.')
-    mod  = '.'.join(backend[:-1])
-    func = backend[-1]
-    try:
-        CONVERTOR_BACKEND = getattr(import_module(mod), func)
-    except:
-        CONVERTOR_BACKEND = None
-else:
-    CONVERTOR_BACKEND = None
-
 class Report(object):
     enable_threads  = True
     create_force    = True
     expiration_time = 86400 # 1 day
-    report_format   = ('pdf', 'application/pdf') # ending and content type
+    mimetype        = 'application/vnd.oasis.opendocument.text'
+    format          = 'fodt'
     icon            = None
-    template_name   = 'reportapi/docs/base.html'
+    template_name   = 'reportapi/flatxml/standard_text.html'
     filters         = None
     site            = None
     name            = None
     title           = None
-    verbose_name    = None
+    verbose_name    = None # do not change
     section         = ugettext_noop('main')
-    section_label   = None
+    section_label   = None # do not change
+    page            = Page()
 
     def __init__(self, site=None, section=None, section_label=None, \
         filters=None, title=None, name=None, **kwargs):
@@ -91,21 +124,24 @@ class Report(object):
                 'English without translation.')
         self.verbose_name = _(self.title)
 
+    def slugify(self, name):
+        return slugify(name)
+
     @property
     def label(self):
         return self.verbose_name
 
-    @property
-    def format(self):
-        return self.report_format[0]
-
     def create_register(self):
         """
         Создаёт в базе данных объект регистрации отчёта
         """
         registers = Register.objects.filter(section=self.section, name=self.name)
         if registers:
-            return registers[0]
+            register = registers[0]
+            if register.title != self.title:
+                register.title = self.title
+                register.save()
+            return register
         # Если данный отчёт не зарегистрирован, то создаём его
         return Register.objects.create(
             section=self.section, name=self.name, title=self.title)
@@ -141,7 +177,7 @@ class Report(object):
         return code.hexdigest()
 
     def get_filename(self):
-        filename = smart_text(self.verbose_name) + '.html'
+        filename = smart_text(self.verbose_name) + '.' + self.format
         filename = prep_filename(filename)
         return filename
 
@@ -154,11 +190,50 @@ class Report(object):
         """
         raise NotImplementedError()
 
+    def get_description_from_filters(self, filters):
+        """
+        Получаем описание из фильтров
+        """
+
+        filters = self.get_filters_data(filters)
+        L = []
+        for f in filters:
+            label = (f['label']).lower()
+            clabel = (f['condition_label']).lower()
+
+            if f['condition'] in ('isnull', 'empty'):
+                cond = _('empty') if f['value'] else _('no empty')
+                s = '%s: %s' % (label, cond)
+            elif f['condition'] == 'in':
+                cond = ', '.join([ smart_text(x) for x in f['value_label']])
+                s = '%s: %s [%s]' % (label, clabel, cond)
+            elif f['condition'] == 'range':
+                cond = [ smart_text(x) for x in f['value_label']]
+                s = _('%(label)s: from %(cond0)s to %(cond1)s') % {
+                    'label': label, 'cond0': cond[0], 'cond1': cond[1]
+                }
+            else:
+                cond = f['value_label']
+                s = '%s: %s %s' % (label, clabel, cond)
+
+            L.append(s)
+        return ';\n'.join(L)
+
+    def get_details(self, request, document, filters):
+        """
+        Этот метод может быть переопределён в наследуемых классах.
+        """
+        return {}
+
     def render(self, request, document, filters, save=False):
         """
         Формирование файла отчёта.
         """
         context = self.get_context(request, document, filters)
+        if not 'BRAND_TEXT' in context:
+            context['BRAND_TEXT'] = REPORTAPI_BRAND_TEXT
+        document.mimetype   = self.mimetype
+        context['PAGE']     = self.page.checked()
         context['DOCUMENT'] = document
         context['FILTERS'] = self.get_filters_data(filters)
         content = loader.render_to_string(self.template_name, context,
@@ -181,10 +256,14 @@ class Report(object):
         return [ x.serialize() for x in self.filters ]
 
     def validate_filters(self, filters):
+        """
+        Raise Exception from filter if not valid
+        """
         try:
             data = self.get_filters_data(filters)
-        except:
-            return False
+        except Exception as e:
+            print filters
+            raise e
         return True
 
     def get_filters_data(self, filters):
@@ -197,7 +276,7 @@ class Report(object):
 
     def get_filter_data(self, name, filters):
         f = self.get_filter(name)
-        kw = filters.get(slugify(name), None)
+        kw = filters.get(self.slugify(name), None)
         if f and kw:
             return f.data(**kw)
         return None
@@ -209,7 +288,7 @@ class Report(object):
         return None
 
     def get_filter(self, name):
-        return self._filters.get(slugify(name), None)
+        return self._filters.get(self.slugify(name), None)
 
     def get_scheme(self, request=None):
         SCHEME = {
@@ -217,7 +296,7 @@ class Report(object):
             'section': self.section,
             'label': self.label,
             'icon': self.icon,
-            'format': self.report_format[0],
+            'format': self.format,
             'enable_threads': self.enable_threads,
             'create_force': self.create_force,
             'expiration_time': self.expiration_time,
@@ -233,6 +312,11 @@ class Report(object):
     def timeout(self):
         return self.create_register().timeout
 
+class Spreadsheet(Report):
+    mimetype      = "application/vnd.oasis.opendocument.spreadsheet"
+    format        = 'fods'
+    template_name = 'reportapi/flatxml/standard_spreadsheet.html'
+
 class RegisterManager(models.Manager):
     use_for_related_fields = True
 
@@ -274,7 +358,7 @@ class Register(models.Model):
 
     def __str__(self):
         try:
-            return smart_text(_(self.title))
+            return ugettext(self.title)
         except:
             return self.title
 
@@ -327,16 +411,19 @@ class Document(models.Model):
     end   = models.DateTimeField(_('end create'), null=True, blank=True)
     report_file = models.FileField(_('report file'), blank=True,
         upload_to=lambda x,y: x.upload_to(y), max_length=512)
+    description = models.TextField(_('description'), blank=True)
+    details = JSONField(_('details'), null=True, blank=True)
 
     objects = DocumentManager()
 
     def __str__(self):
-        return smart_text(self.register)
+        return force_text(self.register)
 
     class Meta:
         ordering = ['-start', '-end']
         verbose_name = _('generated report')
         verbose_name_plural = _('generated reports')
+        get_latest_by = 'end'
 
     def upload_to(self, filename):
         dt = timezone.now()
@@ -347,7 +434,8 @@ class Document(models.Model):
         }
         code = hashlib.new(REPORTAPI_UPLOAD_HASHLIB)
         code.update(str(dt.isoformat()))
-        code.update('reportapi'+settings.SECRET_KEY)
+        code.update('reportapi')
+        code.update(get_random_string())
         dic['code'] = code.hexdigest()
         return smart_text('reports/%(date)s/%(code)s/%(filename)s' % dic)
 
@@ -355,82 +443,82 @@ class Document(models.Model):
     def get_absolute_url(self):
         return ('reportapi:get_document', [self.pk])
 
+    @property
+    def created(self):
+        return self.end
+
     @property
     def url(self):
         if self.report_file:
             return self.report_file.url
         return None
 
-    def format_url(self, format):
-        format = format.lower()
-        if self.report_file:
-            if self.convert_to(format):
-                return self.report_file.url[:-4] + format # cut 'html' and append format file
-        return None
-
-    def convert_to(self, format):
+    def autoconvert(self, remove=True):
         """
-        Конвертирует из HTML в заданный формат. Поддерживаются:
-            1. ODT, ODS (фиктивная конвертация простым изменением
-                        окончания файла)
-            2. PDF (натуральная конвертация с использованием 
-                    настроек REPORTAPI_PDFCONVERT_ARGS[1,2])
+        Конвертирует из FlatXML в заданный формат если на сервере
+        установлен `unoconv`
+        Поддерживаются:
+            1. ODF (ODT, ODS, ODP)
+            2. PDF
 
-        Может использовать внешнюю функцию конвертации, заданную в
-        параметре REPORTAPI_CONVERTOR_BACKEND как строка для импорта.
-        Такая функция должна принимать 2 параметра и возвращать строку
-        пути к файлу, либо `None`:
-
-        def myconvertor(document, format):
-            return path_to_file or None
+        Устанавливает путь к новому файлу относительно
+        расположения хранилища: self.report_file.name
+        
+        Возвращает булево исполнения операции.
         """
-        if CONVERTOR_BACKEND:
-            return CONVERTOR_BACKEND(document=self, format=format)
 
-        format = format.lower()
-        path = self.report_file.path
-        newpath = path[:-4] + format # cut 'html' and append format file
-        if format in ('odt', 'ods'):
-            if not os.path.exists(newpath):
-                cwd = os.getcwd()
-                os.chdir(os.path.dirname(path))
-                os.symlink(os.path.basename(path), os.path.basename(newpath))
-                os.chdir(cwd)
+        location = self.report_file.storage.location
+        oldpath = self.report_file.path
+        oldname = self.report_file.name
+        basename, ext = os.path.splitext(oldname)
+        ext = ext.lower()
 
-        elif format == 'pdf':
-            if not os.path.exists(newpath):
-                cwd = os.getcwd()
+        ExtODF = {'.fodt': '.odt', '.fods': '.ods', '.fodp': '.odp'}
 
-                proc = REPORTAPI_PDFCONVERT_ARGS1 + [
-                    os.path.basename(path)
-                ] + REPORTAPI_PDFCONVERT_ARGS2
-
-                out = "/dev/null"
-                err = "/dev/null"
-                p = subprocess.Popen(proc, shell=False,
-                        stdout=open(out, 'w+b'), 
-                        stderr=open(err, 'w+b'),
-                        cwd=os.path.dirname(path))
-                a = p.wait()
-
-                os.chdir(cwd)
-
-                if not os.path.exists(newpath):
-                    return None
+        if REPORTAPI_UNOCONV_TO_PDF and ext != '.pdf':
+            format = 'pdf'
+            newname = basename + '.pdf'
+        elif REPORTAPI_UNOCONV_TO_ODF and ext in ExtODF:
+            format = ExtODF[ext][1:]
+            newname = basename + ExtODF[ext]
         else:
-            return path
+            return False
 
-        return newpath
+        newpath = os.path.join(location, newname)
+        if remove and os.path.exists(newpath):
+            remove_file(newpath)
+
+        if not os.path.exists(newpath):
+            dwd = os.path.dirname(newpath)
+            cwd = os.getcwd()
+            os.chdir(dwd)
+
+            proc = [UNOCONV_EXE, '-f', format, os.path.basename(oldpath)]
+
+            out = 'out' #if settings.DEBUG else "/dev/null"
+            err = 'err' #if settings.DEBUG else "/dev/null"
+            p = subprocess.Popen(proc, shell=False,
+                    stdout=open(out, 'w+b'), 
+                    stderr=open(err, 'w+b'),
+                    cwd=dwd)
+            a = p.wait()
+
+            os.chdir(cwd)
+
+            if os.path.exists(newpath):
+                self.report_file.name = newname
+                if remove:
+                    remove_file(oldpath)
+                return True
+
+        return False
 
-    def delete(self):
-        remove_dirs(os.path.dirname(self.report_file.path), withfiles=True)
-        super(Document, self).delete()
 
     def save(self):
         if self.id:
             old = Document.objects.get(id=self.id)
             try:
-                presave_obj.report_file.path
+                old.report_file.path
             except:
                 pass
             else:
@@ -440,7 +528,7 @@ class Document(models.Model):
 
     def delete(self):
         if self.report_file:
-            remove_file(self.report_file.path)
+            remove_dirs(os.path.dirname(self.report_file.path), withfiles=True)
         super(Document, self).delete()
 
 
