@@ -40,7 +40,7 @@
 //                   КОНСТАНТЫ И ГЛОБАЛЬНЫЕ ПЕРЕМЕННЫЕ                //
 ////////////////////////////////////////////////////////////////////////
 var TIMEOUT_PROGRESS = 1000,
-    SERVER_TZ_OFFSET = window.SERVER_TZ_OFFSET || -240; // Europe/Moscow
+    SERVER_TZ_OFFSET = window.SERVER_TZ_OFFSET || -180; // Europe/Moscow
 
 // Глобальные хранилища-регистраторы
 window.TEMPLATES = {}; // Шаблоны
@@ -129,9 +129,9 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
          * возврата на текущую страницу
          */
         if ((json.status >=300) && (json.status <400) && (json.data.Location != undefined)) {
-            redirect = function() {
-                location = json.data.Location
-                .replace(/\?.*$/, "?next=" + window.location.pathname);
+            var redirect = function() {
+                var location = json.data.Location
+                    .replace(/\?.*$/, "?next=" + window.location.pathname);
                 window.location.replace(location);
             }
             if (json.message) {
@@ -142,13 +142,17 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
         /* При ошибках извещаем пользователя полученным сообщением */
         else if (json.status >=400) {
             handlerShowAlert(json.message, 'alert-danger');
+            clearInterval(window.REPORT.process);
+            window.REPORT.process = undefined;
+            $('.progress .progress-bar').removeClass('progress-striped active');
+            handlerSetProgress(0);
         }
         /* При нормальном возврате в debug-режиме выводим в консоль
          * сообщение
          */
         else {
             if (DEBUG) {
-                o = new Object;
+                var o = new Object;
                 $.extend(true, o, json.data);
                 //~ console.log(o);
                 console.log($.toJSON(json.message));
@@ -166,8 +170,10 @@ function jsonAPI(args, callback, to_console, sync, timeout) {
 /* Обработчик создания отчёта */
 function handlerCreateReport(force) {
     if (DEBUG) {console.log('function:'+'handlerCreateReport')};
-    $('.action-preview, .action-remove, .action-download')
+    $('.action-create-report, .action-recreate-report, .action-preview, .action-remove')
         .attr('disabled', 'disabled');
+    $('#view-document-place').hide();
+
     window.REPORT.id = null;
     window.REPORT.process_filters = {};
     $.each(window.REPORT.filters, function(key, item) {
@@ -175,16 +181,13 @@ function handlerCreateReport(force) {
             window.REPORT.process_filters[item.name] = {condition:item.condition, value:item.value};
         };
     });
-    args = {
+    var args = {
         'method': 'reportapi.document_create',
         'section': window.REPORT.section,
         'name': window.REPORT.name,
         'filters': window.REPORT.process_filters,
-        'force': force,
-    };
-    cb = function(json, status, xhr) {
-        
-    };
+        'force': force},
+        cb = function(json, status, xhr) {};
 
     var jqxhr = jsonAPI(args, cb, null, false, window.REPORT.timeout);
     handlerStartProgress();
@@ -206,7 +209,7 @@ function eventRecreateReport(event) {
 /* Обработчик запуска ожидания создания отчёта */
 function handlerCheckProcess() {
     if (DEBUG) {console.log('function:'+'handlerCheckProcess')};
-    args = {
+    var args = {
         'method': 'reportapi.document_info',
         'id': window.REPORT.id || null,
         'section': window.REPORT.section,
@@ -214,62 +217,60 @@ function handlerCheckProcess() {
         'filters': window.REPORT.process_filters,
     };
 
-    cb = function(json, status, xhr) {
-        $obj = $('.progress .progress-bar');
-        max = Number($obj.attr('aria-valuemax'));
-        now = Number($obj.attr('aria-valuetransitiongoal'));
+    var cb = function(json, status, xhr) {
+        var $pbar = $('.progress .progress-bar'),
+            max = Number($pbar.attr('aria-valuemax')),
+            now = Number($pbar.attr('aria-valuenow'));
         window.REPORT.id = json.data.id;
         window.REPORT.timeout = json.data.timeout;
         if (json.data.end) {
+            // report ready
             clearInterval(window.REPORT.process);
             window.REPORT.process = undefined;
-            $obj.attr('aria-valuetransitiongoal', max).progressbar();
-            
-            url = json.data.url + window.REPORT.format.toLowerCase() + '/';
+            handlerSetProgress(max);
 
             $('.action-preview')
-                .attr('onClick', "handlerShowDocument('"+json.data.url+"', 'document-"+json.data.id+"')")
-                .removeAttr('disabled').show();
+                .attr('onclick', "handlerShowDocument('"+json.data.url+"', 'document-"+json.data.id+"')")
+                .prop("disabled", false).removeAttr('disabled');
             if (json.data.error) {
-                $('.action-download').attr('disabled', 'disabled').hide();
-                $('.action-remove')
-                    .attr('onClick', "handlerRemoveDocument("+json.data.id+")")
-                    .attr('disabled', 'disabled').show();
-                if (json.data.has_remove) {
-                    $('.action-remove').removeAttr('disabled');
-                };
                 $('.action-preview')
-                    .removeClass('btn-default').addClass('btn-warning')
+                    .removeClass('btn-default btn-success')
+                    .addClass('btn-warning')
                     .find('.fa').removeClass('fa-search').addClass('fa-bug');
             } else {
-                $('.action-remove').attr('disabled', 'disabled').hide();
-                $('.action-download')
-                    .attr('href', url)
-                    .removeAttr('disabled').show();
                 $('.action-preview')
                     .removeClass('btn-warning').addClass('btn-default')
                     .find('.fa').removeClass('fa-bug').addClass('fa-search');
             };
-            $('.progress')
-                .removeClass('progress-striped')
-                .removeClass('active')
-                .hide();
-            $obj.attr('aria-valuetransitiongoal', 0).progressbar();
+            // If user can remove this report
+            if (json.data.has_remove) {
+                $('.action-remove')
+                    .attr('onclick', "handlerRemoveDocument("+json.data.id+")")
+                    .prop("disabled", false).removeAttr('disabled');
+            } else {
+                $('.action-remove').removeAttr('onclick')
+                    .prop("disabled", true).attr('disabled', 'disabled');
+            };
+
+            // TODO: animation or 1-2 seconds after
+            $pbar.removeClass('progress-striped active');
+            //~ handlerSetProgress(0);
+    
+            $('.action-recreate-report').prop("disabled", false).removeAttr('disabled');
 
             if (window.REPORT.create_force) {
                 $('.action-create-report:visible').hide();
                 $('.action-recreate-report:hidden').show();
             } else {
-                $('.action-create-report').prop("disabled", true);
+                $('.action-create-report').prop("disabled", true).attr('disabled', 'disabled');
             };
-            //~ $('.progress').hide(500);
         } else if (now >= max){
-            //~ $obj.attr('aria-valuetransitiongoal', (max/100)*99).progressbar();
-            $('.progress')
-                .addClass('progress-striped')
-                .addClass('active');
+            // report not ready and count request to server >= maximum request
+            $pbar.addClass('progress-striped active');
         } else {
-            $obj.attr('aria-valuetransitiongoal', TIMEOUT_PROGRESS + now).progressbar();
+            // report not ready
+            console.log(TIMEOUT_PROGRESS + now);
+            handlerSetProgress(TIMEOUT_PROGRESS + now);
         };
     };
 
@@ -283,9 +284,7 @@ function handlerStartProgress() {
     $('.action-create-report').prop("disabled", true);
     $('.progress .progress-bar')
         .attr('aria-valuemax', window.REPORT.timeout || TIMEOUT_PROGRESS)
-        .attr('aria-valuetransitiongoal', 1)
-        .progressbar();
-    $('.progress').show();
+        .attr('aria-valuenow', 0).css('width', '0%');
     
     window.REPORT.process = setInterval(function () {
         handlerCheckProcess();
@@ -293,7 +292,7 @@ function handlerStartProgress() {
 };
 
 /* Обработчик удаления отчёта */
-function handlerRemoveDocument(id) {
+function handlerRemoveDocument(id, refresh) {
     if (DEBUG) {console.log('function:'+'handlerRemoveDocument')};
     if (id) {
         var args = {
@@ -301,9 +300,10 @@ function handlerRemoveDocument(id) {
                 'id': id,
             },
             success = function() {
-                $('.action-preview, .action-remove, .action-download')
-                    .attr('disabled', 'disabled').hide();
-                $('#thumbnail-document-'+id).remove();
+                $('#div-document-'+id).remove();
+                $('#view-document-place').hide();
+                handlerAfterChanges();
+                if (refresh) { window.location.reload() };
             };
         new jsonAPI(args, success);
     };
@@ -324,7 +324,7 @@ function eventRemoveDocument(event) {
 /* Обработчик установки поиска-выбора для фильтров объектов */
 function handlerSetSelectizers($box) {
     if (DEBUG) {console.log('function:'+'handlerSetSelectizers')};
-    $box = $box || $('body');
+    var $box = $box || $('body');
 
     $.each($box.find('select[data-type="object"]'), function(index, select) {
 
@@ -384,7 +384,10 @@ function handlerSetSelectizers($box) {
         }
     });
 
-    other = $box.find('select[data-type="choice"], select[data-type="month"], select[data-type="weekday"]');
+    var other = $box.find('select[data-type="choice"],\
+                      select[data-type="month"],\
+                      select[data-type="weekday"],\
+                      select[data-type="period"]');
     $.each(other, function(index, select) {
 
         if (select.name) {
@@ -416,7 +419,7 @@ function handlerSetSelectizers($box) {
 /* Обработчик установки масок для input */
 function handlerMaskInputs($box) {
     if (DEBUG) {console.log('function:'+'handlerMaskInputs')};
-    $box = $box || $('body');
+    var $box = $box || $('body');
 
     // Инициализация маски для дат
     $.mask.definitions['1']='[0-1]';
@@ -424,15 +427,16 @@ function handlerMaskInputs($box) {
     $.mask.definitions['3']='[0-3]';
     $.mask.definitions['5']='[0-5]';
     $.each($box.find('[data-mask]'), function(i, item) {
-        $item = $(item);
-        data = $(item).data();
+        var $item = $(item),
+            data = $(item).data();
         $item.mask(data.mask, {
             completed: function() {
-                filter = REPORT.filters[this.context.name];
+                var filter = REPORT.filters[this.context.name];
                 filter.value = this.val();
                 if (filter.condition == 'range') {
                     filter.value = filter.value.split(RANGE_SPLIT);
                 };
+                if (DEBUG) {console.log('function:'+'handlerMaskInputs:completed; filter=' + filter.value)};
             }
         });
     });
@@ -444,16 +448,16 @@ function handlerMaskInputs($box) {
 function handlerCheckRequiredValue() {
     if (DEBUG) {console.log('function:'+'handlerCheckRequiredValue')};//, event)};
 
-    completed = true;
+    var completed = true;
     $.each(REPORT.filters, function(key, item) {
         if ((item.required) && (item.value === null || item.value === undefined)) {
             completed = false;
         };
     });
     if (completed) {
-        $('.action-create-report').prop("disabled", false);
+        $('.action-create-report').prop("disabled", false).removeAttr("disabled");
     } else {
-        $('.action-create-report').prop("disabled", true);
+        $('.action-create-report').prop("disabled", true).attr('disabled', 'disabled');
     };
     return completed;
 };
@@ -466,15 +470,11 @@ function handlerAfterChanges() {
         window.REPORT.id = null;
         window.REPORT.process = null;
     };
-    $('.progress:visible').hide();
-    $('.progress .progress-bar').attr('aria-valuetransitiongoal', 0)
-        .progressbar();
-    $('.action-preview:visible').hide();
-    $('.action-download:visible').hide();
+    handlerSetProgress(0);
+    $('.action-create-report, .action-preview, .action-remove')
+        .attr('disabled', 'disabled');
     $('.action-recreate-report:visible').hide();
     $('.action-create-report:hidden').show();
-    $('.action-preview, .action-remove, .action-download')
-        .attr('disabled', 'disabled').hide();
     handlerCheckRequiredValue();
 };
 
@@ -484,12 +484,12 @@ function handlerAfterChanges() {
 /* Обработчик события изменения значения фильтра */
 function eventChangeValue(event) {
     if (DEBUG) {console.log('function:'+'eventChangeValue')};//, event)};
-    value = $(event.target).val();
-    filter = REPORT.filters[event.target.name];
+    var value = $(event.target).val(),
+        filter = REPORT.filters[event.target.name];
     if (event.target.type == 'checkbox') {
         filter.value = event.target.checked;
     } else if ($(event.target).attr('data-mask') !== undefined) {
-        filter.value = value ? filter.value : null;
+        filter.value = filter.value || null;
     } else {
         filter.value = value || null;
     };
@@ -500,8 +500,8 @@ function eventChangeValue(event) {
 function eventConditionChange(event) {
     if (DEBUG) {console.log('function:'+'eventConditionChange')};//, event)};
 
-    filter_name = $(event.target).data()['name'];
-    filter = REPORT.filters[filter_name];
+    var filter_name = $(event.target).data()['name'],
+        filter = REPORT.filters[filter_name];
     filter.condition = event.target.value || null;
     //~ console.log(filter.condition);
     if (filter.condition in {'isnull':0, 'empty':0}) {
@@ -511,7 +511,7 @@ function eventConditionChange(event) {
     } else {
         filter.value = $('#value-'+filter_name).val() || null;
     };
-    html = TEMPLATES.filter({ data: filter });
+    var html = TEMPLATES.filter({ data: filter });
     $('#valuebox-'+filter_name).html(html);
     handlerSetSelectizers($('#valuebox-'+filter_name));
     handlerMaskInputs($('#valuebox-'+filter_name));
@@ -539,7 +539,7 @@ function keyDownIsControl(event) {
 function eventKeyDownOnDateTime(event) {
     if (DEBUG) {console.log('function:'+'eventKeyDownOnDateTime')};//, event)};
 
-    prepare = function(event) {
+    var prepare = function(event) {
         value = event.target.value || '';
         //~ console.log(value, event.charCode);
     };
@@ -560,7 +560,7 @@ function eventKeyDownOnDateTime(event) {
 function eventKeyDownOnNumber(event) {
     //~ if (DEBUG) {console.log('function:'+'eventKeyDownOnNumber')};//, event)};
 
-    adddot = function(event) {
+    var adddot = function(event) {
         value = event.target.value || '';
         //~ console.log('search', value.search('.'));
         if (value.length < 1) {
@@ -588,19 +588,43 @@ function eventKeyDownOnNumber(event) {
 //                              ПРОЧЕЕ                                //
 ////////////////////////////////////////////////////////////////////////
 
+/* Обработчик прогрессбара */
+function handlerSetProgress(now) {
+    if (DEBUG) {console.log('function:'+'handlerSetProgress')};
+    var $pbar = $('.progress .progress-bar'),
+        max = Number($pbar.attr('aria-valuemax')) || 100,
+        now = now || 0,
+        width = (now / max) * 100;
+    width = width > 100 ? 100 : width;
+    $pbar.attr('aria-valuenow', now).css('width', width +'%');
+};
+
 /* Обработчик просмотра отчёта */
-function handlerShowDocument(href, name) {
+function handlerShowDocument(href, name, inwindow) {
     if (DEBUG) {console.log('function:'+'handlerShowDocument')};
-    var win = window.open(href, name, 'height=500,width=800,resizable=yes,scrollbars=yes');
-    win.focus();
+    if (inwindow) {
+        var win = window.open(href, name, 'height=500,width=800,resizable=yes,scrollbars=yes');
+        win.focus();
+    } else {
+        var $place  = $('#view-document-place'),
+            html = '<iframe class="embed-responsive-item" src="'
+                   +href
+                   +'" allowfullscreen webkitallowfullscreen></iframe>';
+        if ($place.size() == 0) {
+            $('#view-'+name).toggle();
+            $('#view-'+name+':visible .embed-responsive').html(html);
+        } else {
+            $place.show().find('.embed-responsive').html(html);
+        }
+    }
 };
 
 /* Обработчик отображения дополнительных фильтров */
 function handlerShowAdditionalFilters() {
     if (DEBUG) {console.log('function:'+'handlerShowAdditionalFilters')};
     $('.additional-filter').show();
-    $('button.action-hide-additional').show();
-    $('button.action-show-additional').hide();
+    $('.action-hide-additional').show();
+    $('.action-show-additional').hide();
 
     return true;
 };
@@ -609,8 +633,8 @@ function handlerShowAdditionalFilters() {
 function handlerHideAdditionalFilters() {
     if (DEBUG) {console.log('function:'+'handlerHideAdditionalFilters')};
     $('.additional-filter').hide();
-    $('button.action-hide-additional').hide();
-    $('button.action-show-additional').show();
+    $('.action-hide-additional').hide();
+    $('.action-show-additional').show();
     return true;
 };
 
