@@ -1,4 +1,25 @@
 # -*- coding: utf-8 -*-
+#
+#  reportapi/urls.py
+#  
+#  Copyright 2014 Grigoriy Kramarenko <root@rosix.ru>
+#  
+#  This program is free software; you can redistribute it and/or modify
+#  it under the terms of the GNU General Public License as published by
+#  the Free Software Foundation; either version 3 of the License, or
+#  (at your option) any later version.
+#  
+#  This program is distributed in the hope that it will be useful,
+#  but WITHOUT ANY WARRANTY; without even the implied warranty of
+#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#  GNU General Public License for more details.
+#  
+#  You should have received a copy of the GNU General Public License
+#  along with this program; if not, write to the Free Software
+#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
+#  MA 02110-1301, USA.
+#  
+#  
 from __future__ import unicode_literals
 from django.conf.urls import *
 import os, copy
@@ -25,14 +46,8 @@ urlpatterns = patterns('reportapi.views',
     url(r'^api/', 'api', name='api'),
     url(r'^list/(?P<section>\w+)/$', 'report_list', name='report_list'),
     url(r'^docs/(?P<pk>\d+)/$', 'get_document', name='get_document'),
-    url(r'^docs/(?P<pk>\d+)/(?P<format>\w+)/$', 'get_document', name='get_document_format'),
     url(r'^docs/$', 'documents', name='documents'),
     url(r'^docs/(?P<section>\w+)/$', 'documents', name='documents_section'),
     url(r'^docs/(?P<section>\w+)/(?P<name>\w+)/$', 'documents', name='documents_section_name'),
     url(r'^report/(?P<section>\w+)/(?P<name>\w+)/$', 'report', name='report'),
-    #~ url(r'^test/$', 'test',  name='reportapi_test'),
 )
-
-#~ urlpatterns += patterns('',
-    #~ url(r'^api/',   include('quickapi.urls'), name='reportapi')
-#~ )
